@@ -17,6 +17,7 @@
 #define _ISOC99_SOURCE
 #define _XOPEN_SOURCE
 #define _BSD_SOURCE
+#define _DEFAULT_SOURCE
 #define _XOPEN_SOURCE_EXTENDED	1
 #define _GNU_SOURCE
 
@@ -46,38 +47,37 @@ struct tunnel_list tunnels;
 int rand_source;
 int ppd = 1;                    /* Packet processing delay */
 int control_fd;                 /* descriptor of control area */
-char *args;
 
-char *dial_no_tmp;              /* jz: Dialnumber for Outgoing Call */
+static char *dial_no_tmp;              /* jz: Dialnumber for Outgoing Call */
 int switch_io = 0;              /* jz: Switch for Incoming or Outgoing Call */
 
 static void open_controlfd(void);
 
-volatile sig_atomic_t sigterm_received;
-volatile sig_atomic_t sigint_received;
-volatile sig_atomic_t sigchld_received;
-volatile sig_atomic_t sigusr1_received;;
-volatile sig_atomic_t sighup_received;
+static volatile sig_atomic_t sigterm_received;
+static volatile sig_atomic_t sigint_received;
+static volatile sig_atomic_t sigchld_received;
+static volatile sig_atomic_t sigusr1_received;;
+static volatile sig_atomic_t sighup_received;
 
 struct control_requests_handler {
     char type;
     int (*handler) (FILE* resf, char* bufp);
 };
 
-int control_handle_available(FILE* resf, char* bufp);
-int control_handle_lns_add_modify(FILE* resf, char* bufp);
-int control_handle_lns_status(FILE* resf, char* bufp);
-int control_handle_tunnel(FILE* respf, char* bufp);
-int control_handle_lac_connect(FILE* resf, char* bufp);
-int control_handle_lac_outgoing_call(FILE* resf, char* bufp);
-int control_handle_lac_hangup(FILE* resf, char* bufp);
-int control_handle_lac_disconnect(FILE* resf, char* bufp);
-int control_handle_lac_add_modify(FILE* resf, char* bufp);
-int control_handle_lac_remove(FILE* resf, char* bufp);
-int control_handle_lac_status(FILE* resf, char* bufp);
-int control_handle_lns_remove(FILE* resf, char* bufp);
+static int control_handle_available(FILE* resf, char* bufp);
+static int control_handle_lns_add_modify(FILE* resf, char* bufp);
+static int control_handle_lns_status(FILE* resf, char* bufp);
+static int control_handle_tunnel(FILE* respf, char* bufp);
+static int control_handle_lac_connect(FILE* resf, char* bufp);
+static int control_handle_lac_outgoing_call(FILE* resf, char* bufp);
+static int control_handle_lac_hangup(FILE* resf, char* bufp);
+static int control_handle_lac_disconnect(FILE* resf, char* bufp);
+static int control_handle_lac_add_modify(FILE* resf, char* bufp);
+static int control_handle_lac_remove(FILE* resf, char* bufp);
+static int control_handle_lac_status();
+static int control_handle_lns_remove(FILE* resf, char* bufp);
 
-struct control_requests_handler control_handlers[] = {
+static struct control_requests_handler control_handlers[] = {
     {CONTROL_PIPE_REQ_AVAILABLE, &control_handle_available},
     {CONTROL_PIPE_REQ_LNS_ADD_MODIFY, &control_handle_lns_add_modify},
     {CONTROL_PIPE_REQ_LNS_STATUS, &control_handle_lns_status},
@@ -94,7 +94,7 @@ struct control_requests_handler control_handlers[] = {
     {0, NULL}
 };
 
-void init_tunnel_list (struct tunnel_list *t)
+static void init_tunnel_list (struct tunnel_list *t)
 {
     t->head = NULL;
     t->count = 0;
@@ -102,7 +102,7 @@ void init_tunnel_list (struct tunnel_list *t)
 }
 
 /* Now sends to syslog instead - MvO */
-void show_status (void)
+static void show_status (void)
 {
     struct schedule_entry *se;
     struct tunnel *t;
@@ -207,21 +207,24 @@ void show_status (void)
     l2tp_log (LOG_WARNING, "================================\n");
 }
 
-void null_handler(int sig)
+static void null_handler(int sig)
 {
+	UNUSED(sig);
        /* FIXME
         * A sighup is received when a call is terminated, unknown origine ..
         * I catch it and ll looks good, but ..
         */
 }
 
-void status_handler (int sig)
+static void status_handler (int sig)
 {
+    UNUSED(sig);
     show_status ();
 }
 
-void child_handler (int signal)
+static void child_handler (int sig)
 {
+    UNUSED(sig);
     /*
      * Oops, somebody we launched was killed.
      * It's time to reap them and close that call.
@@ -275,9 +278,13 @@ void child_handler (int signal)
                      * it
                      */
 #ifdef USE_KERNEL
-                 if (!kernel_support)
+                 if (!kernel_support) {
+
 #endif
                     close (c->fd);
+#ifdef USE_KERNEL
+                 }
+#endif
                     c->fd = -1;
                     /*
                      * terminate tunnel and call loops, returning to the
@@ -292,7 +299,7 @@ void child_handler (int signal)
     }
 }
 
-void death_handler (int signal)
+static void death_handler (int signal)
 {
     /*
        * If we get here, somebody terminated us with a kill or a control-c.
@@ -337,28 +344,33 @@ void death_handler (int signal)
     exit (1);
 }
 
-void sigterm_handler(int sig)
+static void sigterm_handler(int sig)
 {
+    UNUSED(sig);
     sigterm_received = 1;
 }
 
-void sigint_handler(int sig)
+static void sigint_handler(int sig)
 {
+    UNUSED(sig);
     sigint_received = 1;
 }
 
-void sigchld_handler(int sig)
+static void sigchld_handler(int sig)
 {
+    UNUSED(sig);
     sigchld_received = 1;
 }
 
-void sigusr1_handler(int sig)
+static void sigusr1_handler(int sig)
 {
+    UNUSED(sig);
     sigusr1_received = 1;
 }
 
-void sighup_handler(int sig)
+static void sighup_handler(int sig)
 {
+    UNUSED(sig);
     sighup_received = 1;
 }
 
@@ -437,18 +449,18 @@ int start_pppd (struct call *c, struct ppp_opts *opts)
        stropt[pos++] = strdup ("plugin");
        stropt[pos++] = strdup ("pppol2tp.so");
        stropt[pos++] = strdup ("pppol2tp");
-       stropt[pos] = (char *) malloc (10);
-       snprintf (stropt[pos], 10, "%d", fd2);
+       stropt[pos] = malloc (11);
+       snprintf (stropt[pos], 11, "%d", fd2);
         pos++;
        if (c->container->lns) {
         stropt[pos++] = strdup ("pppol2tp_lns_mode");
         stropt[pos++] = strdup ("pppol2tp_tunnel_id");
-        stropt[pos] = (char *) malloc (10);
-        snprintf (stropt[pos], 10, "%d", c->container->ourtid);
+        stropt[pos] = malloc (11);
+        snprintf (stropt[pos], 11, "%d", c->container->ourtid);
             pos++;
         stropt[pos++] = strdup ("pppol2tp_session_id");
-        stropt[pos] = (char *) malloc (10);
-        snprintf (stropt[pos], 10, "%d", c->ourcid);
+        stropt[pos] = malloc (11);
+        snprintf (stropt[pos], 11, "%d", c->ourcid);
             pos++;
        }
     }
@@ -682,7 +694,7 @@ void destroy_tunnel (struct tunnel *t)
     free (t);
 }
 
-struct tunnel *l2tp_call (char *host, int port, struct lac *lac,
+static struct tunnel *l2tp_call (char *host, int port, struct lac *lac,
                           struct lns *lns)
 {
     /*
@@ -698,6 +710,14 @@ struct tunnel *l2tp_call (char *host, int port, struct lac *lac,
     {
         l2tp_log (LOG_WARNING, "Host name lookup failed for %s.\n",
              host);
+        if (lac->redial && (lac->rtimeout > 0) && !lac->rsched && lac->active)
+        {
+            struct timeval tv;
+            l2tp_log (LOG_INFO, "Will redial in %d seconds\n", lac->rtimeout);
+            tv.tv_sec = lac->rtimeout;
+            tv.tv_usec = 0;
+            lac->rsched = schedule (tv, magic_lac_dial, lac);
+        }
         return NULL;
     }
     bcopy (hp->h_addr, &addr.s_addr, hp->h_length);
@@ -733,7 +753,7 @@ struct tunnel *l2tp_call (char *host, int port, struct lac *lac,
     return tmp->container;
 }
 
-void magic_lac_tunnel (void *data)
+static void magic_lac_tunnel (void *data)
 {
     struct lac *lac;
     lac = (struct lac *) data;
@@ -759,7 +779,7 @@ void magic_lac_tunnel (void *data)
     }
 }
 
-struct call *lac_call (int tid, struct lac *lac, struct lns *lns)
+static struct call *lac_call (int tid, struct lac *lac, struct lns *lns)
 {
     struct tunnel *t = tunnels.head;
     struct call *tmp;
@@ -826,7 +846,7 @@ void magic_lac_dial (void *data)
     lac_call (lac->t->ourtid, lac, NULL);
 }
 
-void lac_hangup (int cid)
+static void lac_hangup (int cid)
 {
     struct tunnel *t = tunnels.head;
     struct call *tmp;
@@ -853,7 +873,7 @@ void lac_hangup (int cid)
     return;
 }
 
-void lac_disconnect (int tid)
+static void lac_disconnect (int tid)
 {
     struct tunnel *t = tunnels.head;
     while (t)
@@ -920,11 +940,11 @@ struct tunnel *new_tunnel ()
     tmp->txspeed = DEFAULT_TX_BPS;
     memset (tmp->chal_us.reply, 0, MD_SIG_SIZE);
     memset (tmp->chal_them.reply, 0, MD_SIG_SIZE);
-    tmp->chal_them.vector = (unsigned char *) malloc (VECTOR_SIZE);
+    tmp->chal_them.vector = malloc (VECTOR_SIZE);
     return tmp;
 }
 
-void write_res (FILE* res_file, const char *fmt, ...)
+static void write_res (FILE* res_file, const char *fmt, ...)
 {
     if (!res_file || ferror (res_file) || feof (res_file))
         return;
@@ -934,7 +954,7 @@ void write_res (FILE* res_file, const char *fmt, ...)
     va_end (args);
 }
 
-int parse_one_line (char* bufp, int context, void* tc)
+static int parse_one_line (char* bufp, int context, void* tc)
 {
     /* FIXME: I should check for incompatible options */
     char *s, *d, *t;
@@ -991,15 +1011,15 @@ int parse_one_line (char* bufp, int context, void* tc)
     return 0;
 }
 
-int parse_one_line_lac (char* bufp, struct lac *tc){
+static int parse_one_line_lac (char* bufp, struct lac *tc){
     return parse_one_line(bufp, CONTEXT_LAC, tc);
 }
 
-int parse_one_line_lns (char* bufp, struct lns *tc){
+static int parse_one_line_lns (char* bufp, struct lns *tc){
     return parse_one_line(bufp, CONTEXT_LNS, tc);
 }
 
-struct lns* find_lns_by_name(char* name){
+static struct lns* find_lns_by_name(char* name){
     struct lns *cursor;
 
     /* ml: First check to see if we are searching for default */
@@ -1019,7 +1039,8 @@ struct lns* find_lns_by_name(char* name){
     return NULL; /* ml: Ok we could not find anything*/
 }
 
-int control_handle_available(FILE* resf, char* bufp){
+static int control_handle_available(FILE* resf, char* bufp) {
+    UNUSED(bufp);
     struct lac *lac;
     struct lns *lns;
 
@@ -1070,7 +1091,7 @@ int control_handle_available(FILE* resf, char* bufp){
 	return 1;
 }
 
-int control_handle_lns_add_modify(FILE* resf, char* bufp){
+static int control_handle_lns_add_modify(FILE* resf, char* bufp){
     struct lns *lns;
     char* tunstr;
     char delims[] = " ";
@@ -1103,7 +1124,7 @@ int control_handle_lns_add_modify(FILE* resf, char* bufp){
     return 1;
 }
 
-int control_handle_lns_remove(FILE* resf, char* bufp){
+static int control_handle_lns_remove(FILE* resf, char* bufp){
     char *tunstr;
     struct lns* lns;
     struct lns* prev_lns;
@@ -1151,7 +1172,7 @@ int control_handle_lns_remove(FILE* resf, char* bufp){
     return 1;
 }
 
-int control_handle_lns_status(FILE* resf, char* bufp){
+static int control_handle_lns_status(FILE* resf, char* bufp){
     struct lns *lns;
     char* tunstr;
     char delims[] = " ";
@@ -1209,7 +1230,7 @@ int control_handle_lns_status(FILE* resf, char* bufp){
     return 1;
 }
 
-int control_handle_tunnel(FILE* resf, char* bufp){
+static int control_handle_tunnel(FILE* resf, char* bufp){
     char* host;
     host = strchr (bufp, ' ') + 1;
 #ifdef DEBUG_CONTROL
@@ -1223,7 +1244,7 @@ int control_handle_tunnel(FILE* resf, char* bufp){
     return 1;
 }
 
-int control_handle_lac_connect(FILE* resf, char* bufp){
+static int control_handle_lac_connect(FILE* resf, char* bufp){
     char* tunstr = NULL;
     char* authname= NULL;
     char* password = NULL;
@@ -1284,7 +1305,7 @@ int control_handle_lac_connect(FILE* resf, char* bufp){
     return 1;
 }
 
-int control_handle_lac_outgoing_call(FILE* resf, char* bufp){
+static int control_handle_lac_outgoing_call(FILE* resf, char* bufp){
     char* sub_str;
     char* tunstr;
     char* tmp_ptr;
@@ -1339,7 +1360,7 @@ int control_handle_lac_outgoing_call(FILE* resf, char* bufp){
     return 1;
 }
 
-int control_handle_lac_hangup(FILE* resf, char* bufp){
+static int control_handle_lac_hangup(FILE* resf, char* bufp){
     char* callstr;
     int call;
 
@@ -1354,7 +1375,7 @@ int control_handle_lac_hangup(FILE* resf, char* bufp){
     return 1;
 }
 
-int control_handle_lac_disconnect(FILE* resf, char* bufp){
+static int control_handle_lac_disconnect(FILE* resf, char* bufp){
     char* tunstr;
     struct lac* lac;
     int tunl = 0;
@@ -1400,7 +1421,7 @@ int control_handle_lac_disconnect(FILE* resf, char* bufp){
     return 1;
 }
 
-int control_handle_lac_add_modify(FILE* resf, char* bufp){
+static int control_handle_lac_add_modify(FILE* resf, char* bufp){
     char* tunstr;
     struct lac* lac;
     char delims[] = " ";
@@ -1463,7 +1484,7 @@ int control_handle_lac_add_modify(FILE* resf, char* bufp){
     return 1;
 }
 
-int control_handle_lac_remove(FILE* resf, char* bufp){
+static int control_handle_lac_remove(FILE* resf, char* bufp){
     char *tunstr;
     struct lac* lac;
     struct lac* prev_lac;
@@ -1528,7 +1549,7 @@ int control_handle_lac_remove(FILE* resf, char* bufp){
     return 1;
 }
 
-int control_handle_lac_status(FILE* resf, char* bufp){
+static int control_handle_lac_status(){
     show_status ();
     return 1;
 }
@@ -1629,16 +1650,16 @@ void do_control ()
 }
 
 
-void usage(void) {
+static void usage(void) {
     printf("\nxl2tpd version:  %s\n", SERVER_VERSION);
     printf("Usage: xl2tpd [-c <config file>] [-s <secret file>] [-p <pid file>]\n"
-            "              [-C <control file>] [-D] [-l]\n"
+            "              [-C <control file>] [-D] [-l] [-q <tos decimal value for control>]\n"
             "              [-v, --version]\n");
     printf("\n");
     exit(1);
 }
 
-void init_args(int argc, char *argv[])
+static void init_args(int argc, char *argv[])
 {
     int i=0;
 
@@ -1650,6 +1671,7 @@ void init_args(int argc, char *argv[])
     memset(gconfig.configfile,0,STRLEN);
     memset(gconfig.pidfile,0,STRLEN);
     memset(gconfig.controlfile,0,STRLEN);
+    memset(gconfig.controltos,0,STRLEN);
     strncpy(gconfig.altauthfile,ALT_DEFAULT_AUTH_FILE,
             sizeof(gconfig.altauthfile) - 1);
     strncpy(gconfig.altconfigfile,ALT_DEFAULT_CONFIG_FILE,
@@ -1705,6 +1727,19 @@ void init_args(int argc, char *argv[])
                 strncpy(gconfig.controlfile,argv[i],
                         sizeof(gconfig.controlfile) - 1);
         }
+        else if (! strncmp(argv[i],"-q",2)) {
+            if(++i == argc)
+                usage();
+            else {
+            	strncpy(gconfig.controltos,argv[i],
+            	                        sizeof(gconfig.controltos) - 1);
+            	if (atoi(gconfig.controltos)<0 || atoi(gconfig.controltos)>255)
+            	{
+	            	printf ("TOS value %s out of range(0-255)!\n", gconfig.controltos);
+	            	usage();
+                }
+            }
+        }
         else {
             usage();
         }
@@ -1719,7 +1754,7 @@ void init_args(int argc, char *argv[])
 }
 
 
-void daemonize() {
+static void daemonize() {
     int pid=0;
     int i;
 
@@ -1817,7 +1852,7 @@ static void open_controlfd()
     }
 }
 
-void init (int argc,char *argv[])
+static void init (int argc,char *argv[])
 {
     struct lac *lac;
     struct in_addr listenaddr;
@@ -1894,4 +1929,3 @@ int main (int argc, char *argv[])
     network_thread ();
     return 0;
 }
-
